/* 
 * Copyright (C) 2006 Johannes Hausensteiner
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
/* 
 * Filename: adj_mem.c
 *
 * Function: truncate the end of a .mem file (RAM area)
 * 
 *
 * Changelog
 *
 *  1.0  28.Sep.2006   JH   new
 *
 */


#define VER "1.0"

#ifdef WIN32
#include <windows.h>
#endif

#ifdef __MSDOS__
#include <dir.h>             /* MAXPATH */
#endif

#ifdef __linux__
#include <limits.h>          /* PATH_MAX */
#endif

#include <stdio.h>
#include <string.h>


/******************************  Constants  *******************************/
#define BUFLEN 200

#ifdef WIN32
#define PROGNAME "adj_mem.exe"
#elif defined __linux__
#define PROGNAME "adj_mem"
#define MAX_PATH PATH_MAX
#elif defined __MSDOS__
#define PROGNAME "ADJ_MEM.EXE"
#define MAX_PATH MAXPATH
#else
#error "Unsupported platform!"
#endif


/*******************************  Macros  *********************************/


/*******************************  Globals  ********************************/


/**************************  Function Prototypes  *************************/
int atox (char *s);          /* convert string to HEX number              */
#ifdef __linux__
char *strupr (char *s);      /* convert lowercase letters to uppercase    */
#endif

/* main */
/**************************************************************************/
int main (int argc, char *argv[])
/***************************************************************************
* globals  :                                                               *
* called by:                                                               *
* calls    : atohex()                                                      *
*                                                                          *
* Function                                                                 *
* --------                                                                 *
*                                                                          *
* Returns                                                                  *
* -------                                                                  *
* 0 .. OK                                                                  *
***************************************************************************/
{
  FILE *lst, *mem, *bak;
  char lstname[MAX_PATH], memname[MAX_PATH], bakname[MAX_PATH];
  char buf[BUFLEN];
  char *p, *q;
  unsigned int linecnt, addr, end_addr;
  int i;

  printf ("\n%s V%s, %s\n\n", PROGNAME, VER, __DATE__);
  if (argc == 1)
  {
    printf ("Usage: %s filename[.mem]\n", PROGNAME);
    printf ("      .lst file must be in same directroy as .mem file\n\n");
    return 0;
  }

  /* assemble filenames */
  strcpy (memname, argv[1]);
  p = strrchr (memname, '.');
  if (p == NULL)
    strcat (memname, ".mem");

  strcpy (bakname, argv[1]);
  p = strrchr (bakname, '.');
  if (p)
    *p = '\0';
  strcat (bakname, ".bak");

  strcpy (lstname, argv[1]);
  p = strrchr (lstname, '.');
  if (p)
    *p = '\0';
  strcat (lstname, ".lst");

  /* read the last address from .lst file */
  lst = fopen (lstname, "rt");
  if (lst == NULL)
  {
    perror (lstname);
    return 1;
  }

  printf ("Reading last address from file \"%s\" : ", lstname);

  while (!feof(lst))
  {
    fgets (buf, BUFLEN, lst);
    strupr (buf);
    if (strstr (buf, ".END") != NULL)
      break;
  }
  if (feof(lst))
  {
    fprintf (stderr, "\nCannot find .END directive in \"%s\"\n", lstname);
    fclose (lst);
    fclose (mem);
    fclose (bak);
    return 2;
  }
  fclose (lst);

  /* skip line number */
  p = buf;
  while (!isspace(*p))
    p ++;
  /* skip spaces */
  while (isspace(*p))
    p ++;
  /* find end of address */
  q = p;
  while (!isspace(*q))
    q ++;
  *q = '\0';
  
  end_addr = atox (p);
  printf ("0x%4.4X\n", end_addr);
  
  /* rename .mem file to .bak */
  if (rename (memname, bakname))
  {
    perror ("");
    return 1;
  }

  /* open files */ 
  mem = fopen (memname, "wt");
  if (mem == NULL)
  {
    perror (memname);
    return 1;
  }

  bak = fopen (bakname, "rt");
  if (bak == NULL)
  {
    fclose (mem);
    perror (bakname);
    return 1;
  }

  /* copy input up to the last address to output file */
  printf ("Processing file \"%s\" ...     ", memname);

  linecnt = 0;
  while (!feof(bak))
  {
    fgets (buf, BUFLEN, bak);
    q = strchr (buf, ':');
    if (q == NULL)
    {
      fprintf (stderr, "\n%s(%u): illegal line\n", memname, linecnt);
      continue;
    }
    *q = '\0';
    addr = atox (buf);
    *q = ':';
    if (addr + 16 >= end_addr)
    {
      i = 5 + (end_addr - addr) * 3;
      if (end_addr - addr > 8)
        /* compensate for additional space between 7th and 8th byte */
        i ++;
      buf[i] = '\0';
      fputs (buf, mem);
      fputc ('\n', mem);
      break;
    }
    fputs (buf, mem);
    linecnt ++;
  }
  printf ("\b\b\b\b\b%5d lines done.\n", linecnt);
  fclose (bak);
  fclose (mem);

  /* delete .bak file */
  if (unlink (bakname))
  {
    perror (bakname);
    return 1;
  }

  return 0;
}

/* atox */
/**************************************************************************/
int atox (char *s)           /* convert string to HEX number              */
/***************************************************************************
* globals     :                                                            *
* called from : main()                                                     *
* calls       :                                                            *
*                                                                          *
* Function                                                                 *
* --------                                                                 *
* Convert string to HEX number                                             *
*                                                                          *
* Returns                                                                  *
* --------                                                                 *
* number (as integer)                                                      *
***************************************************************************/
{
   int digit, number;

   number = 0;
   while (*s)
   {
     if ('0' <= *s && *s <= '9')
       digit = *s - '0';
     else if ('A' <= *s && *s <= 'F')
       digit = *s - ('A' - 10);
     else if ('a' <= *s && *s <= 'f')
       digit = *s - ('a' - 10);
     number = (number << 4) + digit;
     s ++;
   }
/* printf("Number = %i, 0x%X\n", number, number);*/
   return number;
}

#ifdef __linux__
/* strupr */
/**************************************************************************/
char *strupr (char *s)       /* convert lowercase letters to uppercase    */
/***************************************************************************
* globals     :                                                            *
* called from : main()                                                     *
* calls       :                                                            *
*                                                                          *
* Function                                                                 *
* --------                                                                 *
* Converts lowercase letters in a string to uppercase.                     *
*                                                                          *
* Returns                                                                  *
* --------                                                                 *
* the string                                                               *
***************************************************************************/
{
  while (*s)
  {
    *s = toupper(*s);
    s ++;
  }
}
#endif
